@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2015 Joseph Gaeddert
+ * Copyright (c) 2007 - 2021 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -53,31 +53,32 @@
 #define DEBUG_DOTPROD_RRRF_MMX   0
 
 // internal methods
-void dotprod_rrrf_execute_mmx(dotprod_rrrf _q,
+int dotprod_rrrf_execute_mmx(dotprod_rrrf _q,
+                             float *      _x,
+                             float *      _y);
+int dotprod_rrrf_execute_mmx4(dotprod_rrrf _q,
                               float *      _x,
                               float *      _y);
-void dotprod_rrrf_execute_mmx4(dotprod_rrrf _q,
-                               float *      _x,
-                               float *      _y);
 
 // basic dot product (ordinal calculation)
-void dotprod_rrrf_run(float *      _h,
-                      float *      _x,
-                      unsigned int _n,
-                      float *      _y)
+int dotprod_rrrf_run(float *      _h,
+                     float *      _x,
+                     unsigned int _n,
+                     float *      _y)
 {
     float r=0;
     unsigned int i;
     for (i=0; i<_n; i++)
         r += _h[i] * _x[i];
     *_y = r;
+    return LIQUID_OK;
 }
 
 // basic dot product (ordinal calculation) with loop unrolled
-void dotprod_rrrf_run4(float *      _h,
-                       float *      _x,
-                       unsigned int _n,
-                       float *      _y)
+int dotprod_rrrf_run4(float *      _h,
+                      float *      _x,
+                      unsigned int _n,
+                      float *      _y)
 {
     float r=0;
 
@@ -98,6 +99,7 @@ void dotprod_rrrf_run4(float *      _h,
         r += _h[i] * _x[i];
 
     *_y = r;
+    return LIQUID_OK;
 }
 
 
@@ -110,8 +112,9 @@ struct dotprod_rrrf_s {
     float * h;          // coefficients array
 };
 
-dotprod_rrrf dotprod_rrrf_create(float *      _h,
-                                 unsigned int _n)
+dotprod_rrrf dotprod_rrrf_create_opt(float *      _h,
+                                     unsigned int _n,
+                                     int          _rev)
 {
     dotprod_rrrf q = (dotprod_rrrf)malloc(sizeof(struct dotprod_rrrf_s));
     q->n = _n;
@@ -120,15 +123,29 @@ dotprod_rrrf dotprod_rrrf_create(float *      _h,
     q->h = (float*) _mm_malloc( q->n*sizeof(float), 16);
 
     // set coefficients
-    memmove(q->h, _h, _n*sizeof(float));
+    unsigned int i;
+    for (i=0; i<q->n; i++)
+        q->h[i] = _h[_rev ? q->n-i-1 : i];
 
     // return object
     return q;
 }
 
+dotprod_rrrf dotprod_rrrf_create(float *      _h,
+                                 unsigned int _n)
+{
+    return dotprod_rrrf_create_opt(_h, _n, 0);
+}
+
+dotprod_rrrf dotprod_rrrf_create_rev(float *      _h,
+                                     unsigned int _n)
+{
+    return dotprod_rrrf_create_opt(_h, _n, 1);
+}
+
 // re-create the structured dotprod object
 dotprod_rrrf dotprod_rrrf_recreate(dotprod_rrrf _q,
-                                   float * _h,
+                                   float *      _h,
                                    unsigned int _n)
 {
     // completely destroy and re-create dotprod object
@@ -136,38 +153,49 @@ dotprod_rrrf dotprod_rrrf_recreate(dotprod_rrrf _q,
     return dotprod_rrrf_create(_h,_n);
 }
 
+// re-create the structured dotprod object, coefficients reversed
+dotprod_rrrf dotprod_rrrf_recreate_rev(dotprod_rrrf _q,
+                                       float *      _h,
+                                       unsigned int _n)
+{
+    // completely destroy and re-create dotprod object
+    dotprod_rrrf_destroy(_q);
+    return dotprod_rrrf_create_rev(_h,_n);
+}
 
-void dotprod_rrrf_destroy(dotprod_rrrf _q)
+
+int dotprod_rrrf_destroy(dotprod_rrrf _q)
 {
     _mm_free(_q->h);
     free(_q);
+    return LIQUID_OK;
 }
 
-void dotprod_rrrf_print(dotprod_rrrf _q)
+int dotprod_rrrf_print(dotprod_rrrf _q)
 {
     printf("dotprod_rrrf [mmx, %u coefficients]\n", _q->n);
     unsigned int i;
     for (i=0; i<_q->n; i++)
         printf("%3u : %12.9f\n", i, _q->h[i]);
+    return LIQUID_OK;
 }
 
 // 
-void dotprod_rrrf_execute(dotprod_rrrf _q,
+int dotprod_rrrf_execute(dotprod_rrrf _q,
                           float *      _x,
                           float *      _y)
 {
     // switch based on size
     if (_q->n < 16) {
-        dotprod_rrrf_execute_mmx(_q, _x, _y);
-    } else {
-        dotprod_rrrf_execute_mmx4(_q, _x, _y);
+        return dotprod_rrrf_execute_mmx(_q, _x, _y);
     }
+    return dotprod_rrrf_execute_mmx4(_q, _x, _y);
 }
 
 // use MMX/SSE extensions
-void dotprod_rrrf_execute_mmx(dotprod_rrrf _q,
-                              float *      _x,
-                              float *      _y)
+int dotprod_rrrf_execute_mmx(dotprod_rrrf _q,
+                             float *      _x,
+                             float *      _y)
 {
     // first cut: ...
     __m128 v;   // input vector
@@ -218,12 +246,13 @@ void dotprod_rrrf_execute_mmx(dotprod_rrrf _q,
 
     // set return value
     *_y = total;
+    return LIQUID_OK;
 }
 
 // use MMX/SSE extensions, unrolled loop
-void dotprod_rrrf_execute_mmx4(dotprod_rrrf _q,
-                               float *      _x,
-                               float *      _y)
+int dotprod_rrrf_execute_mmx4(dotprod_rrrf _q,
+                              float *      _x,
+                              float *      _y)
 {
     // first cut: ...
     __m128 v0, v1, v2, v3;
@@ -297,5 +326,6 @@ void dotprod_rrrf_execute_mmx4(dotprod_rrrf _q,
 
     // set return value
     *_y = total;
+    return LIQUID_OK;
 }
 
