@@ -32,6 +32,7 @@
 
 // 
 #define AUTOCORR(name)      LIQUID_CONCAT(autocorr_rrrf,name)
+#define FDELAY(name)        LIQUID_CONCAT(fdelay_rrrf,name)
 #define FFTFILT(name)       LIQUID_CONCAT(fftfilt_rrrf,name)
 #define FIRDECIM(name)      LIQUID_CONCAT(firdecim_rrrf,name)
 #define FIRFARROW(name)     LIQUID_CONCAT(firfarrow_rrrf,name)
@@ -70,6 +71,7 @@
 
 // source files
 #include "autocorr.c"
+#include "fdelay.c"
 #include "fftfilt.c"
 #include "firdecim.c"
 #include "firfarrow.c"
