@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2019 Joseph Gaeddert
+ * Copyright (c) 2007 - 2020 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -45,6 +45,30 @@
 // select filter estimate method
 #define ESTIMATE_REQ_FILTER_LEN_METHOD          (0)
 
+const char * liquid_firfilt_type_str[LIQUID_FIRFILT_NUM_TYPES][2] = {
+    // short,    long name
+    {"unknown",  "unknown"},
+
+    // Nyquist filter prototypes
+    {"kaiser",   "Nyquist Kaiser filter"},
+    {"pm",       "Parks-McClellan filter"},
+    {"rcos",     "raised-cosine filter"},
+    {"fexp",     "flipped exponential"},
+    {"fsech",    "flipped hyperbolic secant"},
+    {"farcsech", "flipped arc-hyperbolic secant"},
+
+    // root-Nyquist filter prototypes
+    {"arkaiser", "root-Nyquist Kaiser (approximate optimum)"},
+    {"rkaiser",  "root-Nyquist Kaiser (true optimum)"},
+    {"rrcos",    "root raised-cosine"},
+    {"hm3",      "harris-Moerder-3 filter"},
+    {"gmsktx",   "GMSK transmit filter"},
+    {"gmskrx",   "GMSK receive filter"},
+    {"rfexp",    "root flipped exponential"},
+    {"rfsech",   "root flipped hyperbolic secant"},
+    {"rfarcsech","root flipped arc-hyperbolic secant"},
+};
+
 // esimate required filter length given transition bandwidth and
 // stop-band attenuation
 //  _df     :   transition bandwidth (0 < _df < 0.5)
@@ -53,11 +77,11 @@ unsigned int estimate_req_filter_len(float _df,
                                      float _As)
 {
     if (_df > 0.5f || _df <= 0.0f) {
-        fprintf(stderr,"error: estimate_req_filter_len(), invalid bandwidth : %f\n", _df);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"estimate_req_filter_len(), invalid bandwidth : %f", _df);
+        return 0;
     } else if (_As <= 0.0f) {
-        fprintf(stderr,"error: estimate_req_filter_len(), invalid stopband level : %f\n", _As);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"estimate_req_filter_len(), invalid stopband level : %f", _As);
+        return 0;
     }
 
     // compute filter length estimate
@@ -164,11 +188,11 @@ float estimate_req_filter_len_Kaiser(float _df,
                                      float _As)
 {
     if (_df > 0.5f || _df <= 0.0f) {
-        fprintf(stderr,"error: estimate_req_filter_len_Kaiser(), invalid bandwidth : %f\n", _df);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"estimate_req_filter_len_Kaiser(), invalid bandwidth : %f", _df);
+        return 0.0f;
     } else if (_As <= 0.0f) {
-        fprintf(stderr,"error: estimate_req_filter_len(), invalid stopband level : %f\n", _As);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"estimate_req_filter_len(), invalid stopband level : %f", _As);
+        return 0.0f;
     }
 
     // compute filter length estimate
@@ -184,11 +208,11 @@ float estimate_req_filter_len_Herrmann(float _df,
                                        float _As)
 {
     if (_df > 0.5f || _df <= 0.0f) {
-        fprintf(stderr,"error: estimate_req_filter_len_Herrmann(), invalid bandwidth : %f\n", _df);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"estimate_req_filter_len_Herrmann(), invalid bandwidth : %f", _df);
+        return 0.0f;
     } else if (_As <= 0.0f) {
-        fprintf(stderr,"error: estimate_req_filter_len(), invalid stopband level : %f\n", _As);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"estimate_req_filter_len(), invalid stopband level : %f", _As);
+        return 0.0f;
     }
 
     // Gaeddert's revisions:
@@ -249,14 +273,14 @@ void liquid_firdes_kaiser(unsigned int _n,
 {
     // validate inputs
     if (_mu < -0.5f || _mu > 0.5f) {
-        fprintf(stderr,"error: liquid_firdes_kaiser(), _mu (%12.4e) out of range [-0.5,0.5]\n", _mu);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"liquid_firdes_kaiser(), _mu (%12.4e) out of range [-0.5,0.5]", _mu);
+        return;
     } else if (_fc < 0.0f || _fc > 0.5f) {
-        fprintf(stderr,"error: liquid_firdes_kaiser(), cutoff frequency (%12.4e) out of range (0, 0.5)\n", _fc);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"liquid_firdes_kaiser(), cutoff frequency (%12.4e) out of range (0, 0.5)", _fc);
+        return;
     } else if (_n == 0) {
-        fprintf(stderr,"error: liquid_firdes_kaiser(), filter length must be greater than zero\n");
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"liquid_firdes_kaiser(), filter length must be greater than zero");
+        return;
     }
 
     // choose kaiser beta parameter (approximate)
@@ -271,7 +295,7 @@ void liquid_firdes_kaiser(unsigned int _n,
         h1 = sincf(2.0f*_fc*t);
 
         // kaiser window
-        h2 = kaiser(i,_n,beta,_mu);
+        h2 = liquid_kaiser(i,_n,beta);
 
         //printf("t = %f, h1 = %f, h2 = %f\n", t, h1, h2);
 
@@ -292,14 +316,14 @@ void liquid_firdes_notch(unsigned int _m,
 {
     // validate inputs
     if (_m < 1 || _m > 1000) {
-        fprintf(stderr,"error: liquid_firdes_notch(), _m (%12u) out of range [1,1000]\n", _m);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"liquid_firdes_notch(), _m (%12u) out of range [1,1000]", _m);
+        return;
     } else if (_f0 < -0.5f || _f0 > 0.5f) {
-        fprintf(stderr,"error: liquid_firdes_notch(), notch frequency (%12.4e) must be in [-0.5,0.5]\n", _f0);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"liquid_firdes_notch(), notch frequency (%12.4e) must be in [-0.5,0.5]", _f0);
+        return;
     } else if (_As <= 0.0f) {
-        fprintf(stderr,"error: liquid_firdes_notch(), stop-band suppression (%12.4e) must be greater than zero\n", _As);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"liquid_firdes_notch(), stop-band suppression (%12.4e) must be greater than zero", _As);
+        return;
     }
 
     // choose kaiser beta parameter (approximate)
@@ -314,7 +338,7 @@ void liquid_firdes_notch(unsigned int _m,
         float p = -cosf(2.0f*M_PI*_f0*((float)(i) - (float)_m));
 
         // window
-        float w = kaiser(i,h_len,beta,0);
+        float w = liquid_kaiser(i,h_len,beta);
 
         // save un-normalized filter
         _h[i] = p*w;
@@ -415,8 +439,7 @@ void liquid_firdes_prototype(liquid_firfilt_type _type,
         liquid_firdes_rfarcsech(_k, _m, _beta, _dt, _h);
         break;
     default:
-        fprintf(stderr,"error: liquid_firdes_prototype(), invalid root-Nyquist filter type '%d'\n", _type);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"liquid_firdes_prototype(), invalid root-Nyquist filter type '%d'", _type);
     }
 }
 
@@ -447,7 +470,7 @@ void liquid_firdes_doppler(unsigned int _n,
         r = 1.5*_K/(_K+1)*cosf(2*M_PI*_fd*t*cosf(_theta));
 
         // Window
-        w = kaiser(i, _n, beta, 0);
+        w = liquid_kaiser(i, _n, beta);
 
         // composite
         _h[i] = (J+r)*w;
@@ -562,7 +585,6 @@ void liquid_filter_isi(float *      _h,
     // compute zero-lag auto-correlation
     float rxx0 = liquid_filter_autocorr(_h,h_len,0);
     //printf("rxx0 = %12.8f\n", rxx0);
-    //exit(1);
 
     unsigned int i;
     float isi_rms = 0.0f;
@@ -595,14 +617,14 @@ float liquid_filter_energy(float *      _h,
 {
     // validate input
     if (_fc < 0.0f || _fc > 0.5f) {
-        fprintf(stderr,"error: liquid_filter_energy(), cut-off frequency must be in [0,0.5]\n");
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"liquid_filter_energy(), cut-off frequency must be in [0,0.5]");
+        return 0.0f;
     } else if (_h_len == 0) {
-        fprintf(stderr,"error: liquid_filter_energy(), filter length must be greater than zero\n");
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"liquid_filter_energy(), filter length must be greater than zero");
+        return 0.0f;
     } else if (_nfft == 0) {
-        fprintf(stderr,"error: liquid_filter_energy(), fft size must be greater than zero\n");
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"liquid_filter_energy(), fft size must be greater than zero");
+        return 0.0f;
     }
 
     // allocate memory for complex phasor
@@ -644,30 +666,15 @@ float liquid_filter_energy(float *      _h,
 // returns filter type based on input string
 int liquid_getopt_str2firfilt(const char * _str)
 {
-    // Generic filter designs
-    if      (strcmp(_str,"kaiser")   ==0) return LIQUID_FIRFILT_KAISER;
-    else if (strcmp(_str,"pm")       ==0) return LIQUID_FIRFILT_PM;
-    
-    // Nyquist filter designs
-    else if (strcmp(_str,"rcos")     ==0) return LIQUID_FIRFILT_RCOS;
-    else if (strcmp(_str,"fexp")     ==0) return LIQUID_FIRFILT_FEXP;
-    else if (strcmp(_str,"fsech")    ==0) return LIQUID_FIRFILT_FSECH;
-    else if (strcmp(_str,"farcsech") ==0) return LIQUID_FIRFILT_FARCSECH;
+    // compare each string to short name
+    unsigned int i;
+    for (i=0; i<LIQUID_FIRFILT_NUM_TYPES; i++) {
+        if (strcmp(_str,liquid_firfilt_type_str[i][0])==0) {
+            return i;
+        }
+    }
 
-    // root-Nyquist filter designs
-    else if (strcmp(_str,"arkaiser") ==0) return LIQUID_FIRFILT_ARKAISER;
-    else if (strcmp(_str,"rkaiser")  ==0) return LIQUID_FIRFILT_RKAISER;
-    else if (strcmp(_str,"rrcos")    ==0) return LIQUID_FIRFILT_RRC;
-    else if (strcmp(_str,"hM3")      ==0) return LIQUID_FIRFILT_hM3;
-    else if (strcmp(_str,"gmsktx")   ==0) return LIQUID_FIRFILT_GMSKTX;
-    else if (strcmp(_str,"gmskrx")   ==0) return LIQUID_FIRFILT_GMSKRX;
-    else if (strcmp(_str,"rfexp")    ==0) return LIQUID_FIRFILT_RFEXP;
-    else if (strcmp(_str,"rfsech")   ==0) return LIQUID_FIRFILT_RFSECH;
-    else if (strcmp(_str,"rfarcsech")==0) return LIQUID_FIRFILT_RFARCSECH;
-
-    // filter type unknown
+    fprintf(stderr,"warning: liquid_getopt_str2firfilt(), unknown/unsupported type: %s\n", _str);
     return LIQUID_FIRFILT_UNKNOWN;
 }
 
-
-
