@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2018 Joseph Gaeddert
+ * Copyright (c) 2007 - 2020 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -96,13 +96,10 @@ IIRFILT() IIRFILT(_create)(TC *         _b,
                            unsigned int _na)
 {
     // validate input
-    if (_nb == 0) {
-        fprintf(stderr,"error: iirfilt_%s_create(), numerator length cannot be zero\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_na == 0) {
-        fprintf(stderr,"error: iirfilt_%s_create(), denominator length cannot be zero\n", EXTENSION_FULL);
-        exit(1);
-    }
+    if (_nb == 0)
+        return liquid_error_config("iirfilt_%s_create(), numerator length cannot be zero", EXTENSION_FULL);
+    if (_na == 0)
+        return liquid_error_config("iirfilt_%s_create(), denominator length cannot be zero", EXTENSION_FULL);
 
     // create structure and initialize
     IIRFILT() q = (IIRFILT()) malloc(sizeof(struct IIRFILT(_s)));
@@ -166,10 +163,8 @@ IIRFILT() IIRFILT(_create_sos)(TC *         _B,
                                unsigned int _nsos)
 {
     // validate input
-    if (_nsos == 0) {
-        fprintf(stderr,"error: iirfilt_%s_create_sos(), filter must have at least one 2nd-order section\n", EXTENSION_FULL);
-        exit(1);
-    }
+    if (_nsos == 0)
+        return liquid_error_config("iirfilt_%s_create_sos(), filter must have at least one 2nd-order section", EXTENSION_FULL);
 
     // create structure and initialize
     IIRFILT() q = (IIRFILT()) malloc(sizeof(struct IIRFILT(_s)));
@@ -382,10 +377,9 @@ IIRFILT() IIRFILT(_create_differentiator)()
 IIRFILT() IIRFILT(_create_dc_blocker)(float _alpha)
 {
     // validate input
-    if (_alpha <= 0.0f) {
-        fprintf(stderr,"error: iirfilt_%s_create_dc_blocker(), filter bandwidth must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    }
+    if (_alpha <= 0.0f)
+        return liquid_error_config("iirfilt_%s_create_dc_blocker(), filter bandwidth must be greater than zero", EXTENSION_FULL);
+
     // compute DC-blocking filter coefficients
     float bf[2] = {1.0f, -1.0f  };
     float af[2] = {1.0f, -1.0f + _alpha};
@@ -405,16 +399,12 @@ IIRFILT() IIRFILT(_create_pll)(float _w,
                                float _K)
 {
     // validate input
-    if (_w <= 0.0f || _w >= 1.0f) {
-        fprintf(stderr,"error: iirfilt_%s_create_pll(), bandwidth must be in (0,1)\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_zeta <= 0.0f || _zeta >= 1.0f) {
-        fprintf(stderr,"error: iirfilt_%s_create_pll(), damping factor must be in (0,1)\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_K <= 0.0f) {
-        fprintf(stderr,"error: iirfilt_%s_create_pll(), loop gain must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    }
+    if (_w <= 0.0f || _w >= 1.0f)
+        return liquid_error_config("iirfilt_%s_create_pll(), bandwidth must be in (0,1)", EXTENSION_FULL);
+    if (_zeta <= 0.0f || _zeta >= 1.0f)
+        return liquid_error_config("iirfilt_%s_create_pll(), damping factor must be in (0,1)", EXTENSION_FULL);
+    if (_K <= 0.0f)
+        return liquid_error_config("iirfilt_%s_create_pll(), loop gain must be greater than zero", EXTENSION_FULL);
 
     // compute loop filter coefficients
     float bf[3];
