@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2018 Joseph Gaeddert
+ * Copyright (c) 2007 - 2020 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -83,22 +83,16 @@ RESAMP() RESAMP(_create)(float        _rate,
                          unsigned int _npfb)
 {
     // validate input
-    if (_rate <= 0) {
-        fprintf(stderr,"error: resamp_%s_create(), resampling rate must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_m == 0) {
-        fprintf(stderr,"error: resamp_%s_create(), filter semi-length must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_fc <= 0.0f || _fc >= 0.5f) {
-        fprintf(stderr,"error: resamp_%s_create(), filter cutoff must be in (0,0.5)\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_As <= 0.0f) {
-        fprintf(stderr,"error: resamp_%s_create(), filter stop-band suppression must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_npfb == 0) {
-        fprintf(stderr,"error: resamp_%s_create(), number of filter banks must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    }
+    if (_rate <= 0)
+        return liquid_error_config("resamp_%s_create(), resampling rate must be greater than zero", EXTENSION_FULL);
+    if (_m == 0)
+        return liquid_error_config("resamp_%s_create(), filter semi-length must be greater than zero", EXTENSION_FULL);
+    if (_fc <= 0.0f || _fc >= 0.5f)
+        return liquid_error_config("resamp_%s_create(), filter cutoff must be in (0,0.5)", EXTENSION_FULL);
+    if (_As <= 0.0f)
+        return liquid_error_config("resamp_%s_create(), filter stop-band suppression must be greater than zero", EXTENSION_FULL);
+    if (_npfb == 0)
+        return liquid_error_config("resamp_%s_create(), number of filter banks must be greater than zero", EXTENSION_FULL);
 
     // allocate memory for resampler
     RESAMP() q = (RESAMP()) malloc(sizeof(struct RESAMP(_s)));
@@ -145,10 +139,8 @@ RESAMP() RESAMP(_create)(float        _rate,
 RESAMP() RESAMP(_create_default)(float _rate)
 {
     // validate input
-    if (_rate <= 0) {
-        fprintf(stderr,"error: resamp_%s_create_default(), resampling rate must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    }
+    if (_rate <= 0)
+        return liquid_error_config("resamp_%s_create_default(), resampling rate must be greater than zero", EXTENSION_FULL);
 
     // det default parameters
     unsigned int m    = 7;
@@ -207,8 +199,8 @@ void RESAMP(_set_rate)(RESAMP() _q,
                        float    _rate)
 {
     if (_rate <= 0) {
-        fprintf(stderr,"error: resamp_%s_set_rate(), resampling rate must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"resamp_%s_set_rate(), resampling rate must be greater than zero", EXTENSION_FULL);
+        return;
     }
 
     // set internal rate
@@ -229,8 +221,8 @@ void RESAMP(_adjust_rate)(RESAMP() _q,
                           float    _gamma)
 {
     if (_gamma <= 0) {
-        fprintf(stderr,"error: resamp_%s_adjust_rate(), resampling adjustment (%12.4e) must be greater than zero\n", EXTENSION_FULL, _gamma);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"resamp_%s_adjust_rate(), resampling adjustment (%12.4e) must be greater than zero", EXTENSION_FULL, _gamma);
+        return;
     }
 
     // adjust internal rate
@@ -248,9 +240,8 @@ void RESAMP(_set_timing_phase)(RESAMP() _q,
                                float    _tau)
 {
     if (_tau < -1.0f || _tau > 1.0f) {
-        fprintf(stderr,"error: resamp_%s_set_timing_phase(), timing phase must be in [-1,1], is %f\n.",
-                EXTENSION_FULL, _tau);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"resamp_%s_set_timing_phase(), timing phase must be in [-1,1], is %f.",EXTENSION_FULL,_tau);
+        return;
     }
 
     // set internal timing phase
@@ -264,9 +255,8 @@ void RESAMP(_adjust_timing_phase)(RESAMP() _q,
                                   float    _delta)
 {
     if (_delta < -1.0f || _delta > 1.0f) {
-        fprintf(stderr,"error: resamp_%s_adjust_timing_phase(), timing phase adjustment must be in [-1,1], is %f\n.",
-                EXTENSION_FULL, _delta);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"resamp_%s_adjust_timing_phase(), timing phase adjustment must be in [-1,1], is %f.",EXTENSION_FULL,_delta);
+        return;
     }
 
     // adjust internal timing phase
@@ -332,8 +322,7 @@ void RESAMP(_execute)(RESAMP()       _q,
             }
             break;
         default:
-            fprintf(stderr,"error: resamp_%s_execute(), invalid/unknown state\n", EXTENSION_FULL);
-            exit(1);
+            liquid_error(LIQUID_EICONFIG,"resamp_%s_execute(), invalid/unknown state", EXTENSION_FULL);
         }
     }
 
