@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2018 Joseph Gaeddert
+ * Copyright (c) 2007 - 2020 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -123,19 +123,14 @@ SYMSYNC() SYMSYNC(_create)(unsigned int _k,
                            unsigned int _h_len)
 {
     // validate input
-    if (_k < 2) {
-        fprintf(stderr,"error: symsync_%s_create(), input sample rate must be at least 2\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_M == 0) {
-        fprintf(stderr,"error: symsync_%s_create(), number of filter banks must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_h_len == 0) {
-        fprintf(stderr,"error: symsync_%s_create(), filter length must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    } else if ( (_h_len-1) % _M ) {
-        fprintf(stderr,"error: symsync_%s_create(), filter length must be of the form: h_len = m*_k*_M + 1 \n", EXTENSION_FULL);
-        exit(1);
-    }
+    if (_k < 2)
+        return liquid_error_config("symsync_%s_create(), input sample rate must be at least 2", EXTENSION_FULL);
+    if (_M == 0)
+        return liquid_error_config("symsync_%s_create(), number of filter banks must be greater than zero", EXTENSION_FULL);
+    if (_h_len == 0)
+        return liquid_error_config("symsync_%s_create(), filter length must be greater than zero", EXTENSION_FULL);
+    if ( (_h_len-1) % _M )
+        return liquid_error_config("symsync_%s_create(), filter length must be of the form: h_len = m*_k*_M + 1 ", EXTENSION_FULL);
 
     // create main object
     SYMSYNC() q = (SYMSYNC()) malloc(sizeof(struct SYMSYNC(_s)));
@@ -212,19 +207,14 @@ SYMSYNC() SYMSYNC(_create_rnyquist)(int          _type,
                                     unsigned int _M)
 {
     // validate input
-    if (_k < 2) {
-        fprintf(stderr,"error: symsync_%s_create_rnyquist(), samples/symbol must be at least 2\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_m == 0) {
-        fprintf(stderr,"error: symsync_%s_create_rnyquist(), filter delay (m) must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_beta < 0.0f || _beta > 1.0f) {
-        fprintf(stderr,"error: symsync_%s_create_rnyquist(), filter excess bandwidth must be in [0,1]\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_M == 0) {
-        fprintf(stderr,"error: symsync_%s_create_rnyquist(), number of filters in bnak must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    }
+    if (_k < 2)
+        return liquid_error_config("symsync_%s_create_rnyquist(), samples/symbol must be at least 2", EXTENSION_FULL);
+    if (_m == 0)
+        return liquid_error_config("symsync_%s_create_rnyquist(), filter delay (m) must be greater than zero", EXTENSION_FULL);
+    if (_beta < 0.0f || _beta > 1.0f)
+        return liquid_error_config("symsync_%s_create_rnyquist(), filter excess bandwidth must be in [0,1]", EXTENSION_FULL);
+    if (_M == 0)
+        return liquid_error_config("symsync_%s_create_rnyquist(), number of filters in bnak must be greater than zero", EXTENSION_FULL);
 
     // allocate memory for filter coefficients
     unsigned int H_len = 2*_M*_k*_m + 1;
@@ -255,19 +245,14 @@ SYMSYNC() SYMSYNC(_create_kaiser)(unsigned int _k,
                                   unsigned int _M)
 {
     // validate input
-    if (_k < 2) {
-        fprintf(stderr,"error: symsync_%s_create_kaiser(), samples/symbol must be at least 2\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_m == 0) {
-        fprintf(stderr,"error: symsync_%s_create_kaiser(), filter delay (m) must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_beta < 0.0f || _beta > 1.0f) {
-        fprintf(stderr,"error: symsync_%s_create_kaiser(), filter excess bandwidth must be in [0,1]\n", EXTENSION_FULL);
-        exit(1);
-    } else if (_M == 0) {
-        fprintf(stderr,"error: symsync_%s_create_kaiser(), number of filters in bnak must be greater than zero\n", EXTENSION_FULL);
-        exit(1);
-    }
+    if (_k < 2)
+        return liquid_error_config("symsync_%s_create_kaiser(), samples/symbol must be at least 2", EXTENSION_FULL);
+    if (_m == 0)
+        return liquid_error_config("symsync_%s_create_kaiser(), filter delay (m) must be greater than zero", EXTENSION_FULL);
+    if (_beta < 0.0f || _beta > 1.0f)
+        return liquid_error_config("symsync_%s_create_kaiser(), filter excess bandwidth must be in [0,1]", EXTENSION_FULL);
+    if (_M == 0)
+        return liquid_error_config("symsync_%s_create_kaiser(), number of filters in bnak must be greater than zero", EXTENSION_FULL);
 
     // allocate memory for filter coefficients
     unsigned int H_len = 2*_M*_k*_m + 1;
@@ -366,8 +351,8 @@ void SYMSYNC(_set_output_rate)(SYMSYNC()    _q,
 {
     // validate input
     if (_k_out == 0) {
-        fprintf(stderr,"error: symsync_%s_output_rate(), output rate must be greater than 0\n", EXTENSION_FULL);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"symsync_%s_output_rate(), output rate must be greater than 0", EXTENSION_FULL);
+        return;
     }
 
     // set output rate
@@ -385,8 +370,8 @@ void SYMSYNC(_set_lf_bw)(SYMSYNC() _q,
 {
     // validate input
     if (_bt < 0.0f || _bt > 1.0f) {
-        fprintf(stderr,"error: symsync_%s_set_lf_bt(), bandwidth must be in [0,1]\n", EXTENSION_FULL);
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"symsync_%s_set_lf_bt(), bandwidth must be in [0,1]", EXTENSION_FULL);
+        return;
     }
 
     // compute filter coefficients from bandwidth
@@ -560,7 +545,7 @@ void SYMSYNC(_output_debug_file)(SYMSYNC()    _q,
 {
     FILE * fid = fopen(_filename, "w");
     if (!fid) {
-        fprintf(stderr,"error: symsync_%s_output_debug_file(), could not open '%s' for writing\n", EXTENSION_FULL, _filename);
+        liquid_error(LIQUID_EICONFIG,"symsync_%s_output_debug_file(), could not open '%s' for writing", EXTENSION_FULL, _filename);
         return;
     }
     fprintf(fid,"%% %s, auto-generated file\n\n", DEBUG_SYMSYNC_FILENAME);
