@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2015 Joseph Gaeddert
+ * Copyright (c) 2007 - 2020 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,22 +33,14 @@
 
 #include "liquid.internal.h"
 
-void dsssframesync_execute_seekpn(dsssframesync _q, liquid_float_complex _x);
-
-int dsssframesync_step(dsssframesync _q, liquid_float_complex _x, liquid_float_complex * _y);
-
-void dsssframesync_execute_rxpreamble(dsssframesync _q, liquid_float_complex _x);
-
-int dsssframesync_decode_header(dsssframesync _q);
-
-int dsssframesync_decode_payload(dsssframesync _q);
-
-void dsssframesync_execute_rxheader(dsssframesync _q, liquid_float_complex _x);
-
-void dsssframesync_execute_rxpayload(dsssframesync _q, liquid_float_complex _x);
-
-void dsssframesync_configure_payload(dsssframesync _q);
-
+int dsssframesync_execute_seekpn    (dsssframesync _q, float complex _x);
+int dsssframesync_step              (dsssframesync _q, float complex _x, float complex * _y);
+int dsssframesync_execute_rxpreamble(dsssframesync _q, float complex _x);
+int dsssframesync_decode_header     (dsssframesync _q);
+int dsssframesync_decode_payload    (dsssframesync _q);
+int dsssframesync_execute_rxheader  (dsssframesync _q, float complex _x);
+int dsssframesync_execute_rxpayload (dsssframesync _q, float complex _x);
+int dsssframesync_configure_payload (dsssframesync _q);
 static dsssframegenprops_s dsssframesyncprops_header_default = {
     DSSSFRAME_H_CRC,
     DSSSFRAME_H_FEC0,
@@ -63,53 +55,53 @@ enum state {
 };
 
 struct dsssframesync_s {
-    framesync_callback callback;
-    void *             userdata;
-    framesyncstats_s   framesyncstats;
-    framedatastats_s   framedatastats;
+    framesync_callback  callback;
+    void *              userdata;
+    framesyncstats_s    framesyncstats;
+    framedatastats_s    framedatastats;
 
-    unsigned int   k;
-    unsigned int   m;
-    float          beta;
-    qdetector_cccf detector;
-    float          tau_hat;
-    float          dphi_hat;
-    float          phi_hat;
-    float          gamma_hat;
-    nco_crcf       mixer;
-    nco_crcf       pll;
+    unsigned int        k;
+    unsigned int        m;
+    float               beta;
+    qdetector_cccf      detector;
+    float               tau_hat;
+    float               dphi_hat;
+    float               phi_hat;
+    float               gamma_hat;
+    nco_crcf            mixer;
+    nco_crcf            pll;
 
-    firpfb_crcf  mf;
-    unsigned int npfb;
-    int          mf_counter;
-    unsigned int pfb_index;
+    firpfb_crcf         mf;
+    unsigned int        npfb;
+    int                 mf_counter;
+    unsigned int        pfb_index;
 
-    liquid_float_complex * preamble_pn;
-    liquid_float_complex * preamble_rx;
-    synth_crcf             header_synth;
-    synth_crcf             payload_synth;
+    float complex *     preamble_pn;
+    float complex *     preamble_rx;
+    synth_crcf          header_synth;
+    synth_crcf          payload_synth;
 
-    int                    header_soft;
-    flexframegenprops_s    header_props;
-    liquid_float_complex * header_spread;
-    unsigned int           header_spread_len;
-    qpacketmodem           header_decoder;
-    unsigned int           header_user_len;
-    unsigned int           header_dec_len;
-    unsigned char *        header_dec;
-    int                    header_valid;
+    int                 header_soft;
+    flexframegenprops_s header_props;
+    float complex *     header_spread;
+    unsigned int        header_spread_len;
+    qpacketmodem        header_decoder;
+    unsigned int        header_user_len;
+    unsigned int        header_dec_len;
+    unsigned char *     header_dec;
+    int                 header_valid;
 
-    int                    payload_soft;
-    liquid_float_complex * payload_spread;
-    unsigned int           payload_spread_len;
-    qpacketmodem           payload_decoder;
-    unsigned int           payload_dec_len;
-    unsigned char *        payload_dec;
-    int                    payload_valid;
+    int                 payload_soft;
+    float complex *     payload_spread;
+    unsigned int        payload_spread_len;
+    qpacketmodem        payload_decoder;
+    unsigned int        payload_dec_len;
+    unsigned char *     payload_dec;
+    int                 payload_valid;
 
-    unsigned int preamble_counter;
-    unsigned int symbol_counter;
-    enum state   state;
+    unsigned int        preamble_counter;
+    unsigned int        symbol_counter;
+    enum state          state;
 };
 
 dsssframesync dsssframesync_create(framesync_callback _callback, void * _userdata)
@@ -123,8 +115,8 @@ dsssframesync dsssframesync_create(framesync_callback _callback, void * _userdat
     q->beta = 0.3f;
 
     unsigned int i;
-    q->preamble_pn = (liquid_float_complex *)calloc(64, sizeof(liquid_float_complex));
-    q->preamble_rx = (liquid_float_complex *)calloc(64, sizeof(liquid_float_complex));
+    q->preamble_pn = (float complex *)calloc(64, sizeof(float complex));
+    q->preamble_rx = (float complex *)calloc(64, sizeof(float complex));
     msequence ms   = msequence_create(7, 0x0089, 1);
     for (i = 0; i < 64; i++) {
         q->preamble_pn[i] = (msequence_advance(ms) ? M_SQRT1_2 : -M_SQRT1_2);
@@ -132,7 +124,7 @@ dsssframesync dsssframesync_create(framesync_callback _callback, void * _userdat
     }
     msequence_destroy(ms);
 
-    liquid_float_complex * pn = (liquid_float_complex *)calloc(64, sizeof(liquid_float_complex));
+    float complex * pn = (float complex *)calloc(64, sizeof(float complex));
     ms                        = msequence_create(7, 0x00cb, 0x53);
     for (i = 0; i < 64; i++) {
         pn[i] = (msequence_advance(ms) ? M_SQRT1_2 : -M_SQRT1_2);
@@ -163,7 +155,7 @@ dsssframesync dsssframesync_create(framesync_callback _callback, void * _userdat
     q->payload_decoder    = qpacketmodem_create();
     q->payload_spread_len = 64;
     q->payload_spread
-        = (liquid_float_complex *)malloc(q->payload_spread_len * sizeof(liquid_float_complex));
+        = (float complex *)malloc(q->payload_spread_len * sizeof(float complex));
 
     dsssframesync_reset_framedatastats(q);
     dsssframesync_reset(q);
@@ -171,11 +163,10 @@ dsssframesync dsssframesync_create(framesync_callback _callback, void * _userdat
     return q;
 }
 
-void dsssframesync_destroy(dsssframesync _q)
+int dsssframesync_destroy(dsssframesync _q)
 {
-    if (!_q) {
-        return;
-    }
+    if (_q == NULL)
+        return liquid_error(LIQUID_EIOBJ,"dsssframesync_destroy(), NULL pointer passed");
 
     free(_q->preamble_pn);
     free(_q->preamble_rx);
@@ -194,15 +185,16 @@ void dsssframesync_destroy(dsssframesync _q)
     synth_crcf_destroy(_q->payload_synth);
 
     free(_q);
+    return LIQUID_OK;
 }
 
-void dsssframesync_print(dsssframesync _q)
+int dsssframesync_print(dsssframesync _q)
 {
     printf("dsssframesync:\n");
-    framedatastats_print(&_q->framedatastats);
+    return framedatastats_print(&_q->framedatastats);
 }
 
-void dsssframesync_reset(dsssframesync _q)
+int dsssframesync_reset(dsssframesync _q)
 {
     qdetector_cccf_reset(_q->detector);
 
@@ -216,6 +208,7 @@ void dsssframesync_reset(dsssframesync _q)
     _q->symbol_counter   = 0;
 
     _q->framesyncstats.evm = 0.f;
+    return LIQUID_OK;
 }
 
 int dsssframesync_is_frame_open(dsssframesync _q)
@@ -223,7 +216,7 @@ int dsssframesync_is_frame_open(dsssframesync _q)
     return (_q->state == DSSSFRAMESYNC_STATE_DETECTFRAME) ? 0 : 1;
 }
 
-void dsssframesync_set_header_len(dsssframesync _q, unsigned int _len)
+int dsssframesync_set_header_len(dsssframesync _q, unsigned int _len)
 {
     _q->header_user_len = _len;
     _q->header_dec_len  = DSSSFRAME_H_DEC + _q->header_user_len;
@@ -237,85 +230,78 @@ void dsssframesync_set_header_len(dsssframesync _q, unsigned int _len)
                            LIQUID_MODEM_BPSK);
 
     _q->header_spread_len = synth_crcf_get_length(_q->header_synth);
-    _q->header_spread     = (liquid_float_complex *)realloc(
-        _q->header_spread, _q->header_spread_len * sizeof(liquid_float_complex));
+    _q->header_spread     = (float complex *)realloc(
+        _q->header_spread, _q->header_spread_len * sizeof(float complex));
+    return LIQUID_OK;
 }
 
-void dsssframesync_decode_header_soft(dsssframesync _q, int _soft)
+int dsssframesync_decode_header_soft(dsssframesync _q, int _soft)
 {
     _q->header_soft = _soft;
+    return LIQUID_OK;
 }
 
-void dsssframesync_decode_payload_soft(dsssframesync _q, int _soft)
+int dsssframesync_decode_payload_soft(dsssframesync _q, int _soft)
 {
     _q->payload_soft = _soft;
+    return LIQUID_OK;
 }
 
 int dsssframesync_set_header_props(dsssframesync _q, dsssframegenprops_s * _props)
 {
-    if (_props == NULL) {
+    if (_props == NULL)
         _props = &dsssframesyncprops_header_default;
-    }
 
-    if (_props->check == LIQUID_CRC_UNKNOWN || _props->check >= LIQUID_CRC_NUM_SCHEMES) {
-        fprintf(
-            stderr, "error: dsssframesync_set_header_props(), invalid/unsupported CRC scheme\n");
-        exit(1);
-    } else if (_props->fec0 == LIQUID_FEC_UNKNOWN || _props->fec1 == LIQUID_FEC_UNKNOWN) {
-        fprintf(
-            stderr, "error: dsssframesync_set_header_props(), invalid/unsupported FEC scheme\n");
-        exit(1);
-    }
+    if (_props->check == LIQUID_CRC_UNKNOWN || _props->check >= LIQUID_CRC_NUM_SCHEMES)
+        return liquid_error(LIQUID_EIMODE,"dsssframesync_set_header_props(), invalid/unsupported CRC scheme");
+    if (_props->fec0 == LIQUID_FEC_UNKNOWN || _props->fec1 == LIQUID_FEC_UNKNOWN)
+        return liquid_error(LIQUID_EIMODE,"dsssframesync_set_header_props(), invalid/unsupported FEC scheme");
 
     // copy properties to internal structure
     memmove(&_q->header_props, _props, sizeof(dsssframegenprops_s));
 
     // reconfigure payload buffers (reallocate as necessary)
-    dsssframesync_set_header_len(_q, _q->header_user_len);
-
-    return 0;
+    return dsssframesync_set_header_len(_q, _q->header_user_len);
 }
 
-void dsssframesync_execute(dsssframesync _q, liquid_float_complex * _x, unsigned int _n)
+int dsssframesync_execute(dsssframesync _q, float complex * _x, unsigned int _n)
 {
     unsigned int i;
     for (i = 0; i < _n; i++) {
         switch (_q->state) {
         case DSSSFRAMESYNC_STATE_DETECTFRAME:
             // detect frame (look for p/n sequence)
-            dsssframesync_execute_seekpn(_q, _x[i]);
-            break;
+            return dsssframesync_execute_seekpn(_q, _x[i]);
         case DSSSFRAMESYNC_STATE_RXPREAMBLE:
             // receive p/n sequence symbols
-            dsssframesync_execute_rxpreamble(_q, _x[i]);
-            break;
+            return dsssframesync_execute_rxpreamble(_q, _x[i]);
         case DSSSFRAMESYNC_STATE_RXHEADER:
             // receive header symbols
-            dsssframesync_execute_rxheader(_q, _x[i]);
-            break;
+            return dsssframesync_execute_rxheader(_q, _x[i]);
         case DSSSFRAMESYNC_STATE_RXPAYLOAD:
             // receive payload symbols
-            dsssframesync_execute_rxpayload(_q, _x[i]);
-            break;
+            return dsssframesync_execute_rxpayload(_q, _x[i]);
         default:
-            fprintf(stderr, "error: dsssframesync_exeucte(), unknown/unsupported state\n");
-            exit(1);
+            return liquid_error(LIQUID_EINT,"dsssframesync_execute(), invalid internal state");
         }
     }
+    return LIQUID_OK;
 }
 
 // execute synchronizer, seeking p/n sequence
 //  _q      :   frame synchronizer object
 //  _x      :   input sample
 //  _sym    :   demodulated symbol
-void dsssframesync_execute_seekpn(dsssframesync _q, float complex _x)
+int dsssframesync_execute_seekpn(dsssframesync _q, float complex _x)
 {
     // push through pre-demod synchronizer
     float complex * v = qdetector_cccf_execute(_q->detector, _x);
 
     // check if frame has been detected
+    printf("seeking pn...\n");
     if (v == NULL)
-        return;
+        return LIQUID_OK;
+    printf("FRAME DETECTED\n");
 
     // get estimates
     _q->tau_hat   = qdetector_cccf_get_tau(_q->detector);
@@ -344,7 +330,7 @@ void dsssframesync_execute_seekpn(dsssframesync _q, float complex _x)
 
     // run buffered samples through synchronizer
     unsigned int buf_len = qdetector_cccf_get_buf_len(_q->detector);
-    dsssframesync_execute(_q, v, buf_len);
+    return dsssframesync_execute(_q, v, buf_len);
 }
 
 int dsssframesync_step(dsssframesync _q, float complex _x, float complex * _y)
@@ -375,16 +361,15 @@ int dsssframesync_step(dsssframesync _q, float complex _x, float complex * _y)
     return sample_available;
 }
 
-void dsssframesync_execute_rxpreamble(dsssframesync _q, float complex _x)
+int dsssframesync_execute_rxpreamble(dsssframesync _q, float complex _x)
 {
     // step synchronizer
     float complex mf_out           = 0.0f;
     int           sample_available = dsssframesync_step(_q, _x, &mf_out);
 
     // compute output if timeout
-    if (!sample_available) {
-        return;
-    }
+    if (!sample_available)
+        return LIQUID_OK;
 
     // save output in p/n symbols buffer
     unsigned int delay = _q->k * _q->m; // delay from matched filter
@@ -397,37 +382,35 @@ void dsssframesync_execute_rxpreamble(dsssframesync _q, float complex _x)
     _q->preamble_counter++;
 
     // update state
-    if (_q->preamble_counter == 64 + delay) {
+    if (_q->preamble_counter == 64 + delay)
         _q->state = DSSSFRAMESYNC_STATE_RXHEADER;
-    }
+
+    return LIQUID_OK;
 }
 
-void dsssframesync_execute_rxheader(dsssframesync _q, liquid_float_complex _x)
+int dsssframesync_execute_rxheader(dsssframesync _q, float complex _x)
 {
-    liquid_float_complex mf_out           = 0.f;
-    int                  sample_available = dsssframesync_step(_q, _x, &mf_out);
+    float complex mf_out           = 0.f;
+    int           sample_available = dsssframesync_step(_q, _x, &mf_out);
 
-    if (!sample_available) {
-        return;
-    }
+    if (!sample_available)
+        return LIQUID_OK;
 
     _q->header_spread[_q->symbol_counter % synth_crcf_get_length(_q->header_synth)] = mf_out;
     ++_q->symbol_counter;
 
-    if (_q->symbol_counter % synth_crcf_get_length(_q->header_synth) != 0) {
-        return;
-    }
+    if (_q->symbol_counter % synth_crcf_get_length(_q->header_synth))
+        return LIQUID_OK;
 
     int header_complete = dsssframesync_decode_header(_q);
 
-    if (!header_complete) {
-        return;
-    }
+    if (!header_complete)
+        return LIQUID_OK;
 
     if (_q->header_valid) {
         _q->symbol_counter = 0;
         _q->state = DSSSFRAMESYNC_STATE_RXPAYLOAD;
-        return;
+        return LIQUID_OK;
     }
 
     // if not taken, header is NOT valid
@@ -448,12 +431,12 @@ void dsssframesync_execute_rxheader(dsssframesync _q, liquid_float_complex _x)
             _q->header_dec, _q->header_valid, NULL, 0, 0, _q->framesyncstats, _q->userdata);
     }
 
-    dsssframesync_reset(_q);
+    return dsssframesync_reset(_q);
 }
 
 int dsssframesync_decode_header(dsssframesync _q)
 {
-    liquid_float_complex prev_corr, corr, next_corr;
+    float complex prev_corr, corr, next_corr;
     nco_crcf_mix_block_down(
         _q->pll, _q->header_spread, _q->header_spread, synth_crcf_get_length(_q->header_synth));
     synth_crcf_despread_triple(_q->header_synth, _q->header_spread, &prev_corr, &corr, &next_corr);
@@ -471,25 +454,19 @@ int dsssframesync_decode_header(dsssframesync _q)
     return 1;
 }
 
-void dsssframesync_configure_payload(dsssframesync _q)
+int dsssframesync_configure_payload(dsssframesync _q)
 {
     _q->header_valid = qpacketmodem_decode_soft_payload(_q->header_decoder, _q->header_dec);
 
-    if (!_q->header_valid) {
-        return;
-    }
+    if (!_q->header_valid)
+        return LIQUID_OK;
 
     unsigned int n = _q->header_user_len;
 
     unsigned int protocol = _q->header_dec[n + 0];
     if (protocol != DSSSFRAME_PROTOCOL) {
-        fprintf(
-            stderr,
-            "warning, dsssframesync_decode_header(), invalid framing protocol %u (expected %u)\n",
-            protocol,
-            DSSSFRAME_PROTOCOL);
         _q->header_valid = 0;
-        return;
+        return liquid_error(LIQUID_EIMODE,"dsssframesync_decode_header(), invalid framing protocol %u (expected %u)\n",protocol,DSSSFRAME_PROTOCOL);
     }
 
     unsigned int payload_dec_len = (_q->header_dec[n + 1] << 8) | (_q->header_dec[n + 2]);
@@ -500,19 +477,15 @@ void dsssframesync_configure_payload(dsssframesync _q)
     unsigned int fec1  = (_q->header_dec[n + 4]) & 0x1f;
 
     if (check == LIQUID_CRC_UNKNOWN || check >= LIQUID_CRC_NUM_SCHEMES) {
-        fprintf(stderr, "warning: dsssframesync_decode_header(), decoded CRC exceeds available\n");
         _q->header_valid = 0;
-        return;
+        return liquid_error(LIQUID_EIMODE,"dsssframesync_decode_header(), decoded CRC exceeds available\n");
     } else if (fec0 == LIQUID_FEC_UNKNOWN || fec0 >= LIQUID_FEC_NUM_SCHEMES) {
-        fprintf(stderr,
-                "warning: dsssframesync_decode_header(), decoded FEC (inner) exceeds available\n");
         _q->header_valid = 0;
-        return;
+        return liquid_error(LIQUID_EIMODE,"dsssframesync_decode_header(), decoded FEC (inner) exceeds available\n");
+        return LIQUID_OK;
     } else if (fec1 == LIQUID_FEC_UNKNOWN || fec1 >= LIQUID_FEC_NUM_SCHEMES) {
-        fprintf(stderr,
-                "warning: dsssframesync_decode_header(), decoded FEC (outer) exceeds available\n");
         _q->header_valid = 0;
-        return;
+        return liquid_error(LIQUID_EIMODE,"dsssframesync_decode_header(), decoded FEC (outer) exceeds available\n");
     }
 
     _q->payload_dec
@@ -521,31 +494,27 @@ void dsssframesync_configure_payload(dsssframesync _q)
         _q->payload_decoder, _q->payload_dec_len, check, fec0, fec1, LIQUID_MODEM_BPSK);
 
     synth_crcf_set_frequency(_q->payload_synth, synth_crcf_get_frequency(_q->header_synth));
-
-    return;
+    return LIQUID_OK;
 }
 
-void dsssframesync_execute_rxpayload(dsssframesync _q, liquid_float_complex _x)
+int dsssframesync_execute_rxpayload(dsssframesync _q, float complex _x)
 {
-    liquid_float_complex mf_out           = 0.f;
+    float complex mf_out           = 0.f;
     int                  sample_available = dsssframesync_step(_q, _x, &mf_out);
 
-    if (!sample_available) {
-        return;
-    }
+    if (!sample_available)
+        return LIQUID_OK;
 
     _q->payload_spread[_q->symbol_counter % synth_crcf_get_length(_q->payload_synth)] = mf_out;
     ++_q->symbol_counter;
 
-    if (_q->symbol_counter % synth_crcf_get_length(_q->payload_synth) != 0) {
-        return;
-    }
+    if (_q->symbol_counter % synth_crcf_get_length(_q->payload_synth) != 0)
+        return LIQUID_OK;
 
     int payload_complete = dsssframesync_decode_payload(_q);
 
-    if (!payload_complete) {
-        return;
-    }
+    if (!payload_complete)
+        return LIQUID_OK;
 
     _q->framesyncstats.check = qpacketmodem_get_crc(_q->payload_decoder);
     _q->framesyncstats.fec0  = qpacketmodem_get_fec0(_q->payload_decoder);
@@ -561,12 +530,12 @@ void dsssframesync_execute_rxpayload(dsssframesync _q, liquid_float_complex _x)
                      _q->userdata);
     }
 
-    dsssframesync_reset(_q);
+    return dsssframesync_reset(_q);
 }
 
 int dsssframesync_decode_payload(dsssframesync _q)
 {
-    liquid_float_complex prev_corr, corr, next_corr;
+    float complex prev_corr, corr, next_corr;
     nco_crcf_mix_block_down(
         _q->pll, _q->payload_spread, _q->payload_spread, synth_crcf_get_length(_q->payload_synth));
     synth_crcf_despread_triple(
@@ -586,9 +555,9 @@ int dsssframesync_decode_payload(dsssframesync _q)
     return 1;
 }
 
-void dsssframesync_reset_framedatastats(dsssframesync _q)
+int dsssframesync_reset_framedatastats(dsssframesync _q)
 {
-    framedatastats_reset(&_q->framedatastats);
+    return framedatastats_reset(&_q->framedatastats);
 }
 
 framedatastats_s dsssframesync_get_framedatastats(dsssframesync _q)
@@ -596,14 +565,17 @@ framedatastats_s dsssframesync_get_framedatastats(dsssframesync _q)
     return _q->framedatastats;
 }
 
-void dsssframesync_debug_enable(dsssframesync _q)
+int dsssframesync_debug_enable(dsssframesync _q)
 {
+    return liquid_error(LIQUID_EIMODE,"dsssframesync_debug_enable(), mode not supported");
 }
 
-void dsssframesync_debug_disable(dsssframesync _q)
+int dsssframesync_debug_disable(dsssframesync _q)
 {
+    return liquid_error(LIQUID_EIMODE,"dsssframesync_debug_disable(), mode not supported");
 }
 
-void dsssframesync_debug_print(dsssframesync _q, const char * _filename)
+int dsssframesync_debug_print(dsssframesync _q, const char * _filename)
 {
+    return liquid_error(LIQUID_EIMODE,"dsssframesync_debug_print(), mode not supported");
 }
