@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2019 Joseph Gaeddert
+ * Copyright (c) 2007 - 2021 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -37,12 +37,10 @@
 #define DEBUG_QDETECTOR_FILENAME     "qdetector_cccf_debug.m"
 
 // seek signal (initial detection)
-void qdetector_cccf_execute_seek(qdetector_cccf _q,
-                                 float complex  _x);
+int qdetector_cccf_execute_seek(qdetector_cccf _q, float complex  _x);
 
 // align signal in time, compute offset estimates
-void qdetector_cccf_execute_align(qdetector_cccf _q,
-                                  float complex  _x);
+int qdetector_cccf_execute_align(qdetector_cccf _q, float complex  _x);
 
 // main object definition
 struct qdetector_cccf_s {
@@ -88,10 +86,8 @@ qdetector_cccf qdetector_cccf_create(float complex * _s,
                                      unsigned int    _s_len)
 {
     // validate input
-    if (_s_len == 0) {
-        fprintf(stderr,"error: qdetector_cccf_create(), sequence length cannot be zero\n");
-        exit(1);
-    }
+    if (_s_len == 0)
+        return liquid_error_config("qdetector_cccf_create(), sequence length cannot be zero");
     
     // allocate memory for main object and set internal properties
     qdetector_cccf q = (qdetector_cccf) malloc(sizeof(struct qdetector_cccf_s));
@@ -158,19 +154,14 @@ qdetector_cccf qdetector_cccf_create_linear(float complex * _sequence,
                                             float           _beta)
 {
     // validate input
-    if (_sequence_len == 0) {
-        fprintf(stderr,"error: qdetector_cccf_create_linear(), sequence length cannot be zero\n");
-        exit(1);
-    } else if (_k < 2 || _k > 80) {
-        fprintf(stderr,"error: qdetector_cccf_create_linear(), samples per symbol must be in [2,80]\n");
-        exit(1);
-    } else if (_m < 1 || _m > 100) {
-        fprintf(stderr,"error: qdetector_cccf_create_linear(), filter delay must be in [1,100]\n");
-        exit(1);
-    } else if (_beta < 0.0f || _beta > 1.0f) {
-        fprintf(stderr,"error: qdetector_cccf_create_linear(), excess bandwidth factor must be in [0,1]\n");
-        exit(1);
-    }
+    if (_sequence_len == 0)
+        return liquid_error_config("qdetector_cccf_create_linear(), sequence length cannot be zero");
+    if (_k < 2 || _k > 80)
+        return liquid_error_config("qdetector_cccf_create_linear(), samples per symbol must be in [2,80]");
+    if (_m < 1 || _m > 100)
+        return liquid_error_config("qdetector_cccf_create_linear(), filter delay must be in [1,100]");
+    if (_beta < 0.0f || _beta > 1.0f)
+        return liquid_error_config("qdetector_cccf_create_linear(), excess bandwidth factor must be in [0,1]");
     
     // create time-domain template
     unsigned int    s_len = _k * (_sequence_len + 2*_m);
@@ -204,19 +195,14 @@ qdetector_cccf qdetector_cccf_create_gmsk(unsigned char * _sequence,
                                           float           _beta)
 {
     // validate input
-    if (_sequence_len == 0) {
-        fprintf(stderr,"error: qdetector_cccf_create_gmsk(), sequence length cannot be zero\n");
-        exit(1);
-    } else if (_k < 2 || _k > 80) {
-        fprintf(stderr,"error: qdetector_cccf_create_gmsk(), samples per symbol must be in [2,80]\n");
-        exit(1);
-    } else if (_m < 1 || _m > 100) {
-        fprintf(stderr,"error: qdetector_cccf_create_gmsk(), filter delay must be in [1,100]\n");
-        exit(1);
-    } else if (_beta < 0.0f || _beta > 1.0f) {
-        fprintf(stderr,"error: qdetector_cccf_create_gmsk(), excess bandwidth factor must be in [0,1]\n");
-        exit(1);
-    }
+    if (_sequence_len == 0)
+        return liquid_error_config("qdetector_cccf_create_gmsk(), sequence length cannot be zero");
+    if (_k < 2 || _k > 80)
+        return liquid_error_config("qdetector_cccf_create_gmsk(), samples per symbol must be in [2,80]");
+    if (_m < 1 || _m > 100)
+        return liquid_error_config("qdetector_cccf_create_gmsk(), filter delay must be in [1,100]");
+    if (_beta < 0.0f || _beta > 1.0f)
+        return liquid_error_config("qdetector_cccf_create_gmsk(), excess bandwidth factor must be in [0,1]");
     
     // create time-domain template using GMSK modem
     unsigned int    s_len = _k * (_sequence_len + 2*_m);
@@ -256,19 +242,14 @@ qdetector_cccf qdetector_cccf_create_cpfsk(unsigned char * _sequence,
                                            int             _type)
 {
     // validate input
-    if (_sequence_len == 0) {
-        fprintf(stderr,"error: qdetector_cccf_create_cpfsk(), sequence length cannot be zero\n");
-        exit(1);
-    } else if (_k < 2 || _k > 80) {
-        fprintf(stderr,"error: qdetector_cccf_create_cpfsk(), samples per symbol must be in [2,80]\n");
-        exit(1);
-    } else if (_m < 1 || _m > 100) {
-        fprintf(stderr,"error: qdetector_cccf_create_cpfsk(), filter delay must be in [1,100]\n");
-        exit(1);
-    } else if (_beta < 0.0f || _beta > 1.0f) {
-        fprintf(stderr,"error: qdetector_cccf_create_cpfsk(), excess bandwidth factor must be in [0,1]\n");
-        exit(1);
-    }
+    if (_sequence_len == 0)
+        return liquid_error_config("qdetector_cccf_create_cpfsk(), sequence length cannot be zero");
+    if (_k < 2 || _k > 80)
+        return liquid_error_config("qdetector_cccf_create_cpfsk(), samples per symbol must be in [2,80]");
+    if (_m < 1 || _m > 100)
+        return liquid_error_config("qdetector_cccf_create_cpfsk(), filter delay must be in [1,100]");
+    if (_beta < 0.0f || _beta > 1.0f)
+        return liquid_error_config("qdetector_cccf_create_cpfsk(), excess bandwidth factor must be in [0,1]");
 
     // create time-domain template using GMSK modem
     unsigned int    s_len = _k * (_sequence_len + 2*_m);
@@ -289,7 +270,7 @@ qdetector_cccf qdetector_cccf_create_cpfsk(unsigned char * _sequence,
     return q;
 }
 
-void qdetector_cccf_destroy(qdetector_cccf _q)
+int qdetector_cccf_destroy(qdetector_cccf _q)
 {
     // free allocated arrays
     free(_q->s         );
@@ -305,9 +286,10 @@ void qdetector_cccf_destroy(qdetector_cccf _q)
 
     // free main object memory
     free(_q);
+    return LIQUID_OK;
 }
 
-void qdetector_cccf_print(qdetector_cccf _q)
+int qdetector_cccf_print(qdetector_cccf _q)
 {
     printf("qdetector_cccf:\n");
     printf("  template length (time):   %-u\n",   _q->s_len);
@@ -315,10 +297,12 @@ void qdetector_cccf_print(qdetector_cccf _q)
     printf("  search range (bins)   :   %-d\n",   _q->range);
     printf("  detection threshold   :   %6.4f\n", _q->threshold);
     printf("  sum{ s^2 }            :   %.2f\n",  _q->s2_sum);
+    return LIQUID_OK;
 }
 
-void qdetector_cccf_reset(qdetector_cccf _q)
+int qdetector_cccf_reset(qdetector_cccf _q)
 {
+    return LIQUID_OK;
 }
 
 void * qdetector_cccf_execute(qdetector_cccf _q,
@@ -349,32 +333,36 @@ void * qdetector_cccf_execute(qdetector_cccf _q,
     return NULL;
 }
 
-// set detection threshold (should be between 0 and 1, good starting point is 0.5)
-void qdetector_cccf_set_threshold(qdetector_cccf _q,
-                                  float          _threshold)
+// get detection threshold
+float qdetector_cccf_get_threshold(qdetector_cccf _q)
 {
-    if (_threshold <= 0.0f || _threshold > 2.0f) {
-        fprintf(stderr,"warning: threshold (%12.4e) out of range; ignoring\n", _threshold);
-        return;
-    }
+    return _q->threshold;
+}
+
+// set detection threshold (should be between 0 and 1, good starting point is 0.5)
+int qdetector_cccf_set_threshold(qdetector_cccf _q,
+                                 float          _threshold)
+{
+    if (_threshold <= 0.0f || _threshold > 2.0f)
+        return liquid_error(LIQUID_EICONFIG,"threshold (%12.4e) out of range; ignoring", _threshold);
 
     // set internal threshold value
     _q->threshold = _threshold;
+    return LIQUID_OK;
 }
 
 // set carrier offset search range
-void qdetector_cccf_set_range(qdetector_cccf _q,
-                              float          _dphi_max)
+int qdetector_cccf_set_range(qdetector_cccf _q,
+                             float          _dphi_max)
 {
-    if (_dphi_max < 0.0f || _dphi_max > 0.5f) {
-        fprintf(stderr,"warning: carrier offset search range (%12.4e) out of range; ignoring\n", _dphi_max);
-        return;
-    }
+    if (_dphi_max < 0.0f || _dphi_max > 0.5f)
+        return liquid_error(LIQUID_EICONFIG,"carrier offset search range (%12.4e) out of range; ignoring", _dphi_max);
 
     // set internal search range
     _q->range = (int)(_dphi_max * _q->nfft / (2*M_PI));
     _q->range = _q->range < 0 ? 0 : _q->range;
     //printf("range: %d / %u\n", _q->range, _q->nfft);
+    return LIQUID_OK;
 }
 
 // get sequence length
@@ -431,8 +419,8 @@ float qdetector_cccf_get_phi(qdetector_cccf _q)
 //
 
 // seek signal (initial detection)
-void qdetector_cccf_execute_seek(qdetector_cccf _q,
-                                 float complex  _x)
+int qdetector_cccf_execute_seek(qdetector_cccf _q,
+                                float complex  _x)
 {
     // write sample to buffer and increment counter
     _q->buf_time_0[_q->counter++] = _x;
@@ -441,7 +429,7 @@ void qdetector_cccf_execute_seek(qdetector_cccf _q,
     _q->x2_sum_1 += crealf(_x)*crealf(_x) + cimagf(_x)*cimagf(_x);
 
     if (_q->counter < _q->nfft)
-        return;
+        return LIQUID_OK;
     
     // reset counter (last half of time buffer)
     _q->counter = _q->nfft/2;
@@ -464,7 +452,7 @@ void qdetector_cccf_execute_seek(qdetector_cccf _q,
         // swap accumulated signal levels
         _q->x2_sum_0 = _q->x2_sum_1;
         _q->x2_sum_1 = 0.0f;
-        return;
+        return LIQUID_OK;
     }
     float g = 1.0f / ((float)(_q->nfft) * g0 * sqrtf(_q->s2_sum));
     
@@ -539,7 +527,7 @@ void qdetector_cccf_execute_seek(qdetector_cccf _q,
         memmove(_q->buf_time_0, _q->buf_time_0 + rxy_index, (_q->nfft - rxy_index)*sizeof(float complex));
         _q->counter = _q->nfft - rxy_index;
 
-        return;
+        return LIQUID_OK;
     }
 #if DEBUG_QDETECTOR_PRINT
     printf(" no detect, rxy = %12.8f, time index=%u, freq. offset=%d\n", rxy_peak, rxy_index, rxy_offset);
@@ -551,17 +539,18 @@ void qdetector_cccf_execute_seek(qdetector_cccf _q,
     // swap accumulated signal levels
     _q->x2_sum_0 = _q->x2_sum_1;
     _q->x2_sum_1 = 0.0f;
+    return LIQUID_OK;
 }
 
 // align signal in time, compute offset estimates
-void qdetector_cccf_execute_align(qdetector_cccf _q,
-                                  float complex  _x)
+int qdetector_cccf_execute_align(qdetector_cccf _q,
+                                 float complex  _x)
 {
     // write sample to buffer and increment counter
     _q->buf_time_0[_q->counter++] = _x;
 
     if (_q->counter < _q->nfft)
-        return;
+        return LIQUID_OK;
 
     //printf("signal is aligned!\n");
 
@@ -682,5 +671,6 @@ void qdetector_cccf_execute_align(qdetector_cccf _q,
     _q->x2_sum_0 = liquid_sumsqcf(_q->buf_time_0, _q->nfft/2);
     _q->x2_sum_1 = 0;
     _q->counter = _q->nfft/2;
+    return LIQUID_OK;
 }
 
