@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2019 Joseph Gaeddert
+ * Copyright (c) 2007 - 2021 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -20,9 +20,7 @@
  * THE SOFTWARE.
  */
 
-//
-// modem_common.c : common utilities specific to precision
-//
+// common utilities specific to precision
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -113,17 +111,22 @@ struct MODEM(_s)
         struct {
             TC * map;           // 32-sample sub-map (first quadrant)
         } sqam128;
+
+        // pi/4 differential QPSK
+        struct {
+            T theta;    // phase state
+        } pi4dqpsk;
     } data;
 
     // modulate function pointer
-    void (*modulate_func)(MODEM() _q,
-                          unsigned int _symbol_in,
-                          TC * _y);
+    int (*modulate_func)(MODEM() _q,
+                         unsigned int _symbol_in,
+                         TC * _y);
 
     // demodulate function pointer
-    void (*demodulate_func)(MODEM() _q,
-                            TC _x,
-                            unsigned int * _symbol_out);
+    int (*demodulate_func)(MODEM() _q,
+                           TC _x,
+                           unsigned int * _symbol_out);
 
     // soft demodulation
     //int demodulate_soft;    // soft demodulation flag
@@ -198,16 +201,15 @@ MODEM() MODEM(_create)(modulation_scheme _scheme)
     case LIQUID_MODEM_ARB128OPT: return MODEM(_create_arb128opt)();
     case LIQUID_MODEM_ARB256OPT: return MODEM(_create_arb256opt)();
     case LIQUID_MODEM_ARB64VT:   return MODEM(_create_arb64vt)();
+    case LIQUID_MODEM_PI4DQPSK:  return MODEM(_create_pi4dqpsk)();
     
     // arbitrary modem
     case LIQUID_MODEM_ARB:
-        fprintf(stderr,"error: modem_create(), cannot create arbitrary modem (LIQUID_MODEM_ARB) without specifying constellation\n");
-        exit(1);
+        return liquid_error_config("modem%s_create(), cannot create arbitrary modem (LIQUID_MODEM_ARB) without specifying constellation", EXTENSION);
 
     // unknown modulation scheme
     default:
-        fprintf(stderr,"error: modem_create(), unknown/unsupported modulation scheme : %u\n", _scheme);
-        exit(1);
+        return liquid_error_config("modem%s_create(), unknown/unsupported modulation scheme : %u",EXTENSION,_scheme);
     }
 
     // should never get to this point, but adding return statment
@@ -231,7 +233,7 @@ MODEM() MODEM(_recreate)(MODEM() _q,
 }
 
 // destroy a modem object
-void MODEM(_destroy)(MODEM() _q)
+int MODEM(_destroy)(MODEM() _q)
 {
     // free symbol map
     if (_q->symbol_map != NULL)
@@ -252,37 +254,40 @@ void MODEM(_destroy)(MODEM() _q)
 
     // free main object memory
     free(_q);
+    return LIQUID_OK;
 }
 
 // print a modem object
-void MODEM(_print)(MODEM() _q)
+int MODEM(_print)(MODEM() _q)
 {
     printf("linear modem:\n");
     printf("    scheme:         %s\n", modulation_types[_q->scheme].name);
     printf("    bits/symbol:    %u\n", _q->m);
+    return LIQUID_OK;
 }
 
 // reset a modem object (only an issue with dpsk)
-void MODEM(_reset)(MODEM() _q)
+int MODEM(_reset)(MODEM() _q)
 {
     _q->r = 1.0f;         // received sample
     _q->x_hat = _q->r;  // estimated symbol
 
-    if ( liquid_modem_is_dpsk(_q->scheme) )
+    if ( liquid_modem_is_dpsk(_q->scheme) ) {
         _q->data.dpsk.phi = 0.0f;  // reset differential PSK phase state
+    } else if ( _q->scheme == LIQUID_MODEM_PI4DQPSK ) {
+        _q->data.pi4dqpsk.theta = 0; // reset phase state
+    }
+    return LIQUID_OK;
 }
 
 // initialize a generic modem object
-void MODEM(_init)(MODEM() _q,
-                  unsigned int _bits_per_symbol)
+int MODEM(_init)(MODEM()      _q,
+                 unsigned int _bits_per_symbol)
 {
-    if (_bits_per_symbol < 1 ) {
-        fprintf(stderr,"error: modem_init(), modem must have at least 1 bit/symbol\n");
-        exit(1);
-    } else if (_bits_per_symbol > MAX_MOD_BITS_PER_SYMBOL) {
-        fprintf(stderr,"error: modem_init(), maximum number of bits per symbol exceeded\n");
-        exit(1);
-    }
+    if (_bits_per_symbol < 1 )
+        return liquid_error(LIQUID_EICONFIG,"modem%s_init(), modem must have at least 1 bit/symbol",EXTENSION);
+    if (_bits_per_symbol > MAX_MOD_BITS_PER_SYMBOL)
+        return liquid_error(LIQUID_EICONFIG,"modem%s_init(), maximum number of bits per symbol exceeded",EXTENSION);
 
     // initialize common elements
     _q->symbol_map = NULL;    // symbol map (LIQUID_MODEM_ARB only)
@@ -299,26 +304,24 @@ void MODEM(_init)(MODEM() _q,
     // soft demodulation
     _q->demod_soft_neighbors = NULL;
     _q->demod_soft_p = 0;
+    return LIQUID_OK;
 }
 
 // initialize symbol map for fast modulation
-void MODEM(_init_map)(MODEM() _q)
+int MODEM(_init_map)(MODEM() _q)
 {
     // validate input
-    if (_q->symbol_map == NULL) {
-        fprintf(stderr,"error: modem_init_map(), symbol map array has not been allocated\n");
-        exit(1);
-    } else if (_q->M == 0 || _q->M > (1<<MAX_MOD_BITS_PER_SYMBOL)) {
-        fprintf(stderr,"error: modem_init_map(), constellation size is out of range\n");
-        exit(1);
-    } else if (_q->modulate_func == NULL) {
-        fprintf(stderr,"error: modem_init_map(), modulation function has not been initialized\n");
-        exit(1);
-    }
+    if (_q->symbol_map == NULL)
+        return liquid_error(LIQUID_EICONFIG,"modem%s_init_map(), symbol map array has not been allocated",EXTENSION);
+    if (_q->M == 0 || _q->M > (1<<MAX_MOD_BITS_PER_SYMBOL))
+        return liquid_error(LIQUID_EICONFIG,"modem%s_init_map(), constellation size is out of range",EXTENSION);
+    if (_q->modulate_func == NULL)
+        return liquid_error(LIQUID_EICONFIG,"modem%s_init_map(), modulation function has not been initialized",EXTENSION);
 
     unsigned int i;
     for (i=0; i<_q->M; i++)
         _q->modulate_func(_q, i, &_q->symbol_map[i]);
+    return LIQUID_OK;
 }
 
 // Generate random symbol
@@ -343,15 +346,13 @@ modulation_scheme MODEM(_get_scheme)(MODEM() _q)
 //  _q          :   modem object
 //  _symbol_in  :   input symbol
 //  _y          :   output sample
-void MODEM(_modulate)(MODEM() _q,
-                      unsigned int _symbol_in,
-                      TC * _y)
+int MODEM(_modulate)(MODEM()      _q,
+                     unsigned int _symbol_in,
+                     TC *         _y)
 {
     // validate input
-    if (_symbol_in >= _q->M) {
-        fprintf(stderr,"error: modem_modulate(), input symbol exceeds constellation size\n");
-        exit(1);
-    }
+    if (_symbol_in >= _q->M)
+        return liquid_error(LIQUID_EICONFIG,"modem%s_modulate(), input symbol exceeds constellation size", EXTENSION);
 
     if (_q->modulate_using_map) {
         // modulate simply using map (look-up table)
@@ -360,45 +361,45 @@ void MODEM(_modulate)(MODEM() _q,
         // invoke method specific to scheme (calculate symbol on the fly)
         _q->modulate_func(_q, _symbol_in, _y);
     }
+    return LIQUID_OK;
 }
 
 // modulate using symbol map (look-up table)
-void MODEM(_modulate_map)(MODEM() _q,
-                          unsigned int _symbol_in,
-                          TC * _y)
+int MODEM(_modulate_map)(MODEM()      _q,
+                         unsigned int _symbol_in,
+                         TC *         _y)
 {
-    if (_symbol_in >= _q->M) {
-        fprintf(stderr,"error: modem_modulate_table(), input symbol exceeds maximum\n");
-        exit(1);
-    } else if (_q->symbol_map == NULL) {
-        fprintf(stderr,"error: modem_modulate_table(), symbol table not initialized\n");
-        exit(1);
-    }
+    if (_symbol_in >= _q->M)
+        return liquid_error(LIQUID_EICONFIG,"modem%s_modulate_table(), input symbol exceeds maximum", EXTENSION);
+    if (_q->symbol_map == NULL)
+        return liquid_error(LIQUID_EICONFIG,"modem%s_modulate_table(), symbol table not initialized", EXTENSION);
 
     // map sample directly to output
     *_y = _q->symbol_map[_symbol_in]; 
+    return LIQUID_OK;
 }
 
 // generic demodulation
-void MODEM(_demodulate)(MODEM() _q,
-                        TC x,
-                        unsigned int *symbol_out)
+int MODEM(_demodulate)(MODEM() _q,
+                       TC x,
+                       unsigned int *symbol_out)
 {
     // invoke method specific to scheme (calculate symbol on the fly)
-    _q->demodulate_func(_q, x, symbol_out);
+    return _q->demodulate_func(_q, x, symbol_out);
 }
 
 // generic soft demodulation
-void MODEM(_demodulate_soft)(MODEM() _q,
-                             TC _x,
-                             unsigned int  * _s,
-                             unsigned char * _soft_bits)
+int MODEM(_demodulate_soft)(MODEM() _q,
+                            TC _x,
+                            unsigned int  * _s,
+                            unsigned char * _soft_bits)
 {
     // switch scheme
     switch (_q->scheme) {
-    case LIQUID_MODEM_ARB:  MODEM(_demodulate_soft_arb)( _q,_x,_s,_soft_bits); return;
-    case LIQUID_MODEM_BPSK: MODEM(_demodulate_soft_bpsk)(_q,_x,_s,_soft_bits); return;
-    case LIQUID_MODEM_QPSK: MODEM(_demodulate_soft_qpsk)(_q,_x,_s,_soft_bits); return;
+    case LIQUID_MODEM_ARB:  return MODEM(_demodulate_soft_arb)( _q,_x,_s,_soft_bits);
+    case LIQUID_MODEM_BPSK: return MODEM(_demodulate_soft_bpsk)(_q,_x,_s,_soft_bits);
+    case LIQUID_MODEM_QPSK: return MODEM(_demodulate_soft_qpsk)(_q,_x,_s,_soft_bits);
+    case LIQUID_MODEM_PI4DQPSK: return MODEM(_demodulate_soft_pi4dqpsk)(_q,_x,_s,_soft_bits);
     default:;
     }
 
@@ -406,9 +407,7 @@ void MODEM(_demodulate_soft)(MODEM() _q,
     if (_q->demod_soft_neighbors != NULL && _q->demod_soft_p != 0) {
         // demodulate using approximate log-likelihood method with
         // look-up table for nearest neighbors
-        MODEM(_demodulate_soft_table)(_q, _x, _s, _soft_bits);
-
-        return;
+        return MODEM(_demodulate_soft_table)(_q, _x, _s, _soft_bits);
     }
 
     // for now demodulate normally and simply copy the
@@ -418,7 +417,7 @@ void MODEM(_demodulate_soft)(MODEM() _q,
     *_s = symbol_out;
 
     // unpack soft bits
-    liquid_unpack_soft_bits(symbol_out, _q->m, _soft_bits);
+    return liquid_unpack_soft_bits(symbol_out, _q->m, _soft_bits);
 }
 
 #if DEBUG_DEMODULATE_SOFT
@@ -437,10 +436,10 @@ void print_bitstring_demod_soft(unsigned int _x,
 //  _r          :   received sample
 //  _s          :   hard demodulator output
 //  _soft_bits  :   soft bit ouput (approximate log-likelihood ratio)
-void MODEM(_demodulate_soft_table)(MODEM() _q,
-                                   TC _r,
-                                   unsigned int * _s,
-                                   unsigned char * _soft_bits)
+int MODEM(_demodulate_soft_table)(MODEM() _q,
+                                  TC _r,
+                                  unsigned int * _s,
+                                  unsigned char * _soft_bits)
 {
     // run hard demodulation; this will store re-modulated sample
     // as internal variable x_hat
@@ -512,15 +511,17 @@ void MODEM(_demodulate_soft_table)(MODEM() _q,
 
     // set hard output symbol
     *_s = s;
+    return LIQUID_OK;
 }
 
 
 
 // get demodulator's estimated transmit sample
-void MODEM(_get_demodulator_sample)(MODEM() _q,
+int MODEM(_get_demodulator_sample)(MODEM() _q,
                                     TC * _x_hat)
 {
     *_x_hat = _q->x_hat;
+    return LIQUID_OK;
 }
 
 // get demodulator phase error
@@ -541,11 +542,11 @@ T MODEM(_get_demodulator_evm)(MODEM() _q)
 //  _alpha  :   scaling factor
 //  _s      :   demodulated symbol
 //  _res    :   residual
-void MODEM(_demodulate_linear_array)(T              _v,
-                                     unsigned int   _m,
-                                     T              _alpha,
-                                     unsigned int * _s,
-                                     T *            _res)
+int MODEM(_demodulate_linear_array)(T              _v,
+                                    unsigned int   _m,
+                                    T              _alpha,
+                                    unsigned int * _s,
+                                    T *            _res)
 {
     unsigned int s=0;
     unsigned int i, k = _m;
@@ -559,6 +560,7 @@ void MODEM(_demodulate_linear_array)(T              _v,
     }
     *_s = s;
     *_res = _v;
+    return LIQUID_OK;
 }
 
 // Demodulate a linear symbol constellation using refereneced lookup table
@@ -567,11 +569,11 @@ void MODEM(_demodulate_linear_array)(T              _v,
 //  _ref    :   array of thresholds
 //  _s      :   demodulated symbol
 //  _res    :   residual
-void MODEM(_demodulate_linear_array_ref)(T              _v,
-                                         unsigned int   _m,
-                                         T *            _ref,
-                                         unsigned int * _s,
-                                         T *            _res)
+int MODEM(_demodulate_linear_array_ref)(T              _v,
+                                        unsigned int   _m,
+                                        T *            _ref,
+                                        unsigned int * _s,
+                                        T *            _res)
 {
     // initialize loop counter
     register unsigned int i;
@@ -599,18 +601,17 @@ void MODEM(_demodulate_linear_array_ref)(T              _v,
 
     // return residual
     *_res = _v;
+    return LIQUID_OK;
 }
 
 
 // generate soft demodulation look-up table
-void MODEM(_demodsoft_gentab)(MODEM()      _q,
-                              unsigned int _p)
+int MODEM(_demodsoft_gentab)(MODEM()      _q,
+                             unsigned int _p)
 {
     // validate input: ensure number of nearest symbols is not too large
-    if (_p > (_q->M-1)) {
-        fprintf(stderr,"error: modem_demodsoft_gentab(), requesting too many neighbors\n");
-        exit(1);
-    }
+    if (_p > (_q->M-1))
+        return liquid_error(LIQUID_EICONFIG,"modem%s_demodsoft_gentab(), requesting too many neighbors", EXTENSION);
     
     // allocate internal memory
     _q->demod_soft_p = _p;
@@ -625,7 +626,7 @@ void MODEM(_demodsoft_gentab)(MODEM()      _q,
     unsigned int M = _q->M;  // constellation size
     TC c[M];         // constellation
     for (i=0; i<M; i++)
-        modem_modulate(_q, i, &c[i]);
+        MODEM(_modulate)(_q, i, &c[i]);
 
     // 
     // find nearest symbols (see algorithm in sandbox/modem_demodulate_soft_gentab.c)
@@ -706,6 +707,7 @@ void MODEM(_demodsoft_gentab)(MODEM()      _q,
         }
     }
 #endif
+    return LIQUID_OK;
 }
 
 
